# JTFTP - Python/AsyncIO TFTP Server
# Copyright (C) 2022 Jeffrey C. Ollie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


import logging
from typing import Protocol

logger = logging.getLogger(__name__)

from enum import Enum


class FileMode(str, Enum):
    BINARY_READ = "rb"
    BINARY_WRITE = "wb"


class FileProtocol(Protocol):
    async def length(self) -> int | None:
        """Return the length of the file, or None if the length cannot be determined."""
        raise NotImplementedError

    async def seek(self, offset: int, whence: int) -> int:
        raise NotImplementedError

    async def read(self, length: int) -> bytes:
        raise NotImplementedError

    async def write(self, data: bytes) -> int:
        raise NotImplementedError

    async def close(self, complete: bool) -> None:
        raise NotImplementedError


class FilesystemProtocol(Protocol):
    async def open(self, filename: bytes, mode: FileMode) -> FileProtocol:
        raise NotImplementedError
